<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="color-scheme" content="light dark" />
    <meta name="generator" content="ToDo Merger {{ app_version }}">
    <link rel="stylesheet" href="{{ url_for('static', filename='css/pico.min.css') }}" />
    <link rel="stylesheet" href="{{ url_for('static', filename='fontawesome/fontawesome-all.min.css') }}">
    <link rel="stylesheet" href="{{ url_for('static', filename='css/custom.scss.css') }}" />
    <title>ToDo Merger</title>
  </head>
  <body>
    <main class="container">
      {% block content %}
      {% endblock %}
    </main>
  </body>
</html>
