.issue {
  padding: 10px 0 10px 0;
  display: flex;
  flex-wrap: nowrap;
  column-gap: 10px;

  .rank {
    align-content: center;

    a {
      color: #666;

      @media (prefers-color-scheme: light) {
        color: #bbb;
      }

      &.pin {
        &:hover {
          color: deepskyblue;
        }

        &.enabled {
          color: deepskyblue;
        }
      }

      &.high {
        &:hover {
          color: green;
        }

        &.enabled {
          color: green;
        }
      }

      &.low {
        &:hover {
          color: red;
        }

        &.enabled {
          color: red;
        }
      }
    }
  }

  .title {
    font-weight: 700;

    icon {
      margin-right: 5px;
    }
  }

  .metadata {
    font-size: smaller;

    span {
      margin-right: 10px;
    }
  }
}

#refresh-button {
  padding: 5px 10px;
}
